//---------------------------------------------------------------------------

#include "buffer_mixer.h"

//---------------------------------------------------------------------------

CBufferMixer::CBufferMixer()
{
    buffer_size		=	SMP_FREQ;
    buffer_final_size	=	SMP_FREQ;
    reset();
};
//CBufferMixer::CBufferMixer()

char	CBufferMixer::addBuffer(	SMP_TYPE*	new_buffer)
{
	if ( new_buffer == NULL || buffer_last >= SMP_BUFFERS_MAX)
    {
    	say("CBufferMixer::addBuffer - wrong param (%d, %d).",buffer_last,new_buffer);
        return _ERROR;
    }
    buffers[buffer_last++]	=	new_buffer;
    return _OK;
};
//char	CBufferMixer::addBuffer(	SMP_TYPE*	new_buffer)

void		CBufferMixer::reset()
{
	buffer_last = 0;
    delay_on	=	distortion_on	=	filter_on	=	false;
    delay_level	=	delay_time	=	0;
    distortion_max	=  distortion_mirror	=	0;
	filter_freq		=  filter_reso	=	0;
};
//void		CBufferMixer::reset()

void    CBufferMixer::render()
{
	SMP_TYPE*	buf		=  	&buffer[0];
	SMP_TYPE*	p_dst	=	buf;
    SMP_TYPE	tmp;
    // mix all buffers first
	for(int i=0;i<buffer_size;i++,p_dst++)
    {
    	// clean at first (could do in constr. but here's the
        // cycle anyway)
    	*p_dst	=	0;
    	for(int j=0;j<buffer_last;j++)
        {
	    	*p_dst	+=	*(buffers[j]+i);
        }
    }
    // now post FX :
    // delay:
    if (delay_on)
	{
        int			delay_time_scaled	=	( delay_time * buffer_size ) / SMP_DELAY_TIME_MAX;
        SMP_TYPE	delay_level_scaled	=	delay_level/float(SMP_DELAY_LEVEL_MAX);
        p_dst		=	buf + delay_time_scaled;
        SMP_TYPE*	p_src 	=	buf;
        for (int i=delay_time_scaled;i<buffer_size;i++,p_dst++,p_src++)
        {
                tmp		=	*(p_dst) + delay_level_scaled*( *p_src );
                *p_dst	=	 fabs(tmp) > SMP_AMP ? sgn(tmp)*SMP_AMP : tmp;
        }
    }
	// 2nd order band pass filter:
	if (filter_on)
	{
      float reso = filter_reso/float(SMP_FILTER_RESO_MAX+1);

      float	phx = cosf(2 * _PI * filter_freq / buffer_size);
      float	a0 = (1.f-reso) * sqrtf(reso*(reso-4*phx+2)+1.f);
      float	b1 = 2.f*reso*phx;
      float	b2 = -reso*reso;
      p_dst  	=	buf;
      for (int i=0;i<buffer_size;i++)
      {
      	// first 2 ticks die
		*(p_dst++) = i<2 ? 0 : a0 * (*p_dst) + b1 * (*(p_dst-1)) + b2 * (*(p_dst-2));
      }
  	}
    //distortion:
    if (distortion_on)
    {
        float	d_max = distortion_max * SMP_AMP / float(SMP_DISTORTION_MAX);
        float	d_mirror = distortion_mirror * SMP_AMP / float(SMP_DISTORTION_MAX);

        p_dst  	=	buf;
        for (int i=0;i<buffer_size;i++)
        {    
			SMP_TYPE tmp = *p_dst;
			if (fabs(tmp) > d_max)
            {                
                float	mx = ( fabs(tmp)-d_max)/(SMP_AMP-d_max);
                float	mir = (d_max - d_mirror);
                *p_dst = sgn(tmp) * (d_max - (mir*mx));
            }
            p_dst++;
        }
    }
};
//void    CBufferMixer::render();

void	CBufferMixer::prepareFinalBuffer()
{
	SMP_TYPE_FIN*	dst_buf	=	&buffer_final[0];
	SMP_TYPE*		src_buf	=	&buffer[0];
	for(int i=0;i<buffer_size;i++)
    {
    	*dst_buf++	= abs(*src_buf) >= SMP_AMP ? sgn(*src_buf++)* SMP_AMP_FIN :
        					(* src_buf++) * SMP_AMP_FIN;
    }
};
//void	CBufferMixer::prepareFinalBuffer();

SMP_TYPE*	CBufferMixer::getBuffer()
{
    return buffer;
};
//SMP_TYPE*	CBufferMixer::getBuffer();

SMP_TYPE_FIN*	CBufferMixer::getBufferFinal()
{
    return buffer_final;
};
//SMP_TYPE_FIN*	CBufferMixer::getBufferFinal();

int		CBufferMixer::getBufferSize()
{
	return buffer_final_size;
};
//int		CBufferMixer::getBufferSize();

int		CBufferMixer::getBufferFinalSize()
{
	return sizeof(SMP_TYPE_FIN) * buffer_final_size;
};
//int		CBufferMixer::getBufferFinalSize();

