//---------------------------------------------------------------------------

#include "sample_manager.h"

//---------------------------------------------------------------------------

CSampleManager::CSampleManager()
{
	reset();
};
//CSampleManager::CSampleManager()


char	CSampleManager::reset()
{
	// need it when loading new samples to render
	envelope_number	=	buffer_number	=	0;
    for(int i=0;i<SMP_ENVELOPES_MAX;i++)
    {
    	envelopes[i].reset();
        envelopes[i].calcBuffer();
    }
    for(i=0;i<SMP_BUFFERS_MAX;i++)
    {
    	buffers[i].reset();
        buffers[i].generate();
    }
    mixer.reset();
	return	_OK;

};
//char	CSampleManager::reset()

// get 1 elem from command list
SMP_CMD_TYPE	CSampleManager::readElem()
{
	return *cur_position++;
};
//SMP_CMD_TYPE	CSampleManager::readElem()

char	CSampleManager::writeElem(SMP_CMD_TYPE value)
{
	byte*	p_b	=	new byte;
    *p_b	=	value;
  	fwrite(p_b,1,1,f);
	return _OK;
};
//SMP_CMD_TYPE	CSampleManager::readElem()

// gets something like list of commands for sound to be
// generated
char	CSampleManager::generate(	SMP_CMD_TYPE*     new_list)
{
	reset();
	cur_position	=	new_list;
    SMP_CMD_TYPE	tmp;
    // 1-b - version
    version	=	readElem();
    // 1-b - 4bits - env. num.; 4bits - buf. num.
    tmp	=	readElem();

    envelope_number		=	tmp >> 4;
    buffer_number	=	tmp & 0x0f;
    //9-b per envelope
    for(int i=0;i<=envelope_number;i++)
    {
        CEnvelope*	env	=	&envelopes[i];
        for(int j=0;j<ENV_PARAM_LAST;j++)env->params[j]=readElem();
        env->calcBuffer();
    }
    mixer.reset();
    // 5-b per buffer
    for(i=0;i<=buffer_number;i++)
    {
    	CSampleGenerator*	buf	=	&buffers[i];
        // 1 byte :
        //       1 bit - empy ; 1 bit - etmpy
        //       1 bit - amp. modulated? 1 bit - freq. modulate?
        // 		 4 bits -generator func num
        tmp				=	readElem();
        buf->func_num	=	tmp & 0x0f;
        buf->amp_mod	=	bool(tmp & 0x40);
        buf->freq_mod	=	bool(tmp & 0x20);

        // 1-b - amplitude
        buf->amp		=	readElem();
        // 1-b - MSB of freq
        tmp		=	readElem();
        // 1-b - LSB of freq
        buf->freq		=	int(tmp<<8) + readElem();
        //1-b - 4 bits amp. envelope num; 4 bits freq. envelope num;
        tmp	=	readElem();
        buf->amp_env_num	=	buf->amp_mod ? tmp >> 4: 0;
        buf->freq_env_num	=	buf->freq_mod? tmp & 0x0f: 0;

        buf->generate(	buf->amp_mod ? envelopes[buf->amp_env_num].getBuffer() : NULL,
				        buf->freq_mod ? envelopes[buf->freq_env_num].getBuffer() : NULL
        				);

        mixer.addBuffer(buf->getBuffer());
    }
    if( version == SAMPLE_MANAGER_VERSION )
    {
    	// 1-b;
        // 5 bits empty
        // 1bit - delay on
        // 1bit - distortion on
        // 1bit - filter on
        tmp	=	readElem();
        mixer.delay_on		=	tmp & 0x04;
        mixer.distortion_on	=	tmp & 0x02;
        mixer.filter_on		=	tmp & 0x01;
        //2 bytes for delay
        if (mixer.delay_on)
        {
        	// 1b
            mixer.delay_level	=	readElem();
            // 1b
            mixer.delay_time	=	readElem();
        }
        //2 bytes for distortion
        if (mixer.distortion_on)
        {
        	// 1b - distortion limit level
            mixer.distortion_max	=	readElem();
            // 1b - distortion mirror value
            mixer.distortion_mirror	=	readElem();
        }
        //3 bytes for filter
        if (mixer.filter_on)
        {
        	// 2b - freq
            tmp	=	readElem();
            mixer.filter_freq	=	int(tmp<<8) + readElem();
            // 1b - reso
            mixer.filter_reso	=	readElem();
        }
    }
	return _OK;
};
//char	CSampleManager::generate(SMP_CMD_TYPE*     new_list)

#ifdef		_ENABLE_SAVE_SAMPLES
// save DSL
char	CSampleManager::saveFile(char* name)
{
	byte	list_tmp[1000];

    f	=	fopen(name,"wb");

	cur_position	=	&list_tmp[0];
    SMP_CMD_TYPE	tmp;
    // 1-b - version
    writeElem(SAMPLE_MANAGER_VERSION);
    // 1-b - 4bits - env. num.; 4bits - buf. num.
    writeElem(( envelope_number<<4 ) + buffer_number);
    //9-b per envelope
    for(int i=0;i<=envelope_number;i++)
    {
        CEnvelope*	env	=	&envelopes[i];
        for(int j=0;j<ENV_PARAM_LAST;j++)writeElem(env->params[j]);
    }
    // 5-b per buffer
    for(i=0;i<=buffer_number;i++)
    {
    	CSampleGenerator*	buf	=	&buffers[i];
        // 1 byte :
        //       1 bit - empty ; 1 bit - empty
        //       1 bit - amp. modulated? 1 bit - freq. modulate?
        // 		 4 bits -generator func num
        writeElem( 	buf->func_num +
        			((buf->amp_mod)<<6) +
                    ((buf->freq_mod)<<5)
        			);
        // 1-b - amplitude
        writeElem(buf->amp);
        // 1-b - MSB of freq
        // 1-b - LSB of freq
        writeElem(buf->freq >> 8);
        writeElem(buf->freq & 0xff);
        //1-b - 4 bits amp. envelope num; 4 bits freq. envelope num;
        writeElem( (buf->amp_env_num<<4)
        			+ buf->freq_env_num
                    );
    }//for(int i=0;i<=buffer_number;i++)

	// 1-b;
	// 5 bits empty
	// 1bit - delay on
    // 1bit - distortion on
    // 1bit - filter on
    writeElem(	(mixer.delay_on << 2) +
				(mixer.distortion_on << 1) +
				mixer.filter_on );
    //2 bytes for delay
    if (mixer.delay_on)
    {
			// 1b
            writeElem(mixer.delay_level);
            // 1b
            writeElem(mixer.delay_time	);
    }
        //2 bytes for distortion
	if (mixer.distortion_on)
	{
            // 1b - distortion limit level
            writeElem(mixer.distortion_max);
            // 1b - distortion mirror value
            writeElem(mixer.distortion_mirror);
	}
        //3 bytes for filter
	if (mixer.filter_on)
	{
            // 2b - freq
            writeElem(mixer.filter_freq >> 8);
            writeElem(mixer.filter_freq & 0xff);
            writeElem(mixer.filter_reso);
    }

    fclose(f);
    return _OK;
};
//char	CSampleManager::saveFile
#endif

// get buffer ready for playback (e.g. can pass it to/ miniFMOD)
// funcs)
SMP_TYPE_FIN*	CSampleManager::getSample()
{
	return mixer.getBufferFinal();
};
//SMP_TYPE_FINAL*	CSampleManager::getSample()

int	CSampleManager::getSampleSize()
{
	return mixer.getBufferFinalSize();
};
//int	CSampleManager::getSampleSize()