// Test app demonstrating usage of files produced by "DeGenerate"  
// It generates wave files out of "*.dsl"-s stored in current dir.

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include "degen\types.h"
#include "degen\sample_manager.h"

// max size of DSL sample gen. file
#define	SAMPLE_MAX_SIZE			1000l

// for file routines
HANDLE h_file; 
// file name wihout extension (xyz.dsl -> xyz)
char	wav_name[MAX_PATH];

char*	wav_ext	=	{"wav"};

// header for WAV file
struct CWavHead{
  char	riff[4];
  int	size;
  char	wave[4];
  char	fmt[4];
  int	num;
  word	format;
  word	channels;
  int	freq;
  int	abps;
  word	blockalign;
  word	bitdepth;
  char	data[4];
  int	datasize;
};

// header of 44100Hz x 16 bits/sample x Mono  WAV file
CWavHead wave = {
	  { 'R', 'I', 'F', 'F' },
	  sizeof(wave) + 44100*2,
	  { 'W', 'A', 'V', 'E' },
	  { 'f', 'm', 't', ' ' },
	  0x10,
	  1,
	  1,
	  44100,
	  88200,
	  2,
	  16,
	  { 'd', 'a', 't', 'a' },
	  44100*2
};

void	saveWAV(void*	data,	int	size)
{
	
	h_file = CreateFile(wav_name,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if (h_file == INVALID_HANDLE_VALUE) 
	{ 
	    printf("Could not open file (error %d)\n", GetLastError());
	    return;
	}
	DWORD	bytes_written;

	if (! WriteFile(h_file,&wave,sizeof(wave),&bytes_written,NULL) )
	{
		printf("Error during WAV writing!");
		return;
	}
	if (! WriteFile(h_file,data,size,&bytes_written,NULL) )
	{
		printf("Error during WAV writing!");
		return;
	}

	CloseHandle(h_file);
	
}

int main(int argc, char* argv[])
{
	byte*	buffer	=	new byte[SAMPLE_MAX_SIZE];
	
	WIN32_FIND_DATA file_data; 
	HANDLE h_search; 
	bool	more_files	=	true;

	if ((h_search = FindFirstFile("*.dsl", &file_data)) == INVALID_HANDLE_VALUE) 
	{
		printf("No samples found!");
		return 0;
	}
	
	while (more_files)
	{

		int i;
		// copy file name till '.' (extension)
		for (i=0;file_data.cFileName[i] != '.';i++)wav_name[i]=file_data.cFileName[i];
		wav_name[i]	=	0;		
		wsprintf(wav_name,"%s.%s",wav_name,wav_ext);		
		
		h_file = CreateFile(file_data.cFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL); 
		if (h_file == INVALID_HANDLE_VALUE) 
		{ 
			printf("Could not open file (error %d)\n", GetLastError());
			return 0;
		}

		DWORD	bytes_read;
		
		if (! ReadFile(h_file, buffer,file_data.nFileSizeLow,&bytes_read, NULL)) 
		{ 
			printf("Error during reading (error %d)\n", GetLastError());
			return 0;        
		} 

		CloseHandle(h_file);

		// now DSL-file is in buffer and we can generate WAV

		CSampleManager	smp_manager;

		// get all params from DSL
		smp_manager.generate(buffer);
		// render sample
		smp_manager.mixer.render();
		// converts to 16 bit MONO
		smp_manager.mixer.prepareFinalBuffer();
		// and save the sample in WAV-file
		saveWAV(smp_manager.getSample(),smp_manager.getSampleSize());

		// get another DSL
		more_files = FindNextFile(h_search, &file_data);
	}

	FindClose(h_search);

	return 0;	
}